@@ -71,42 +71,14 @@
 }
 #endif
 
-/* There was a very bad bug in 5.6.0 through 5.6.6 downgrading
- * METHOD_SSLv23 to SSLv2 and SSLv3 exclusively */
-#if PHP_VERSION_ID >= 50600 && PHP_VERSION_ID < 50607
-# define PHONGO_CRYPTO_METHOD STREAM_CRYPTO_METHOD_ANY_CLIENT
-#else
-# define PHONGO_CRYPTO_METHOD STREAM_CRYPTO_METHOD_SSLv23_CLIENT
-#endif
-
 #if defined(__GNUC__)
 #  define ARG_UNUSED  __attribute__ ((unused))
 #else
 #  define ARG_UNUSED
 #endif
 
-#ifdef HAVE_ATOLL
-# define STRTOLL(s) atoll(s)
-#else
-# if defined(PHP_WIN32)
-#  define STRTOLL(s) _atoi64(s)
-# else
-#  define STRTOLL(s) strtoll(s, NULL, 10)
-# endif
-#endif
-
-#ifndef php_ignore_value
-# if defined(__GNUC__) && __GNUC__ >= 4
-#  define php_ignore_value(x) (({ __typeof__ (x) __x = (x); (void) __x; }))
-# else
-#  define php_ignore_value(x) ((void) (x))
-# endif
-#endif
-
 #if PHP_VERSION_ID >= 70000
 # define phongo_char zend_string
-# define phongo_char_pdup(str) pestrdup(filename->val, 1)
-# define phongo_char_free(str) zend_string_release(str)
 # define phongo_long zend_long
 #if SIZEOF_ZEND_LONG == 8
 #  define PHONGO_LONG_FORMAT PRId64
@@ -116,18 +88,16 @@
 #  error Unsupported architecture (integers are neither 32-bit nor 64-bit)
 #endif
 # define SIZEOF_PHONGO_LONG SIZEOF_ZEND_LONG
-# define phongo_str(str) str->val
 # define phongo_create_object_retval zend_object*
 # define PHONGO_ALLOC_OBJECT_T(_obj_t, _class_type) (_obj_t *)ecalloc(1, sizeof(_obj_t)+zend_object_properties_size(_class_type))
 # define PHONGO_TSRMLS_FETCH_FROM_CTX(user_data)
 # define SUPPRESS_UNUSED_WARNING(x)
 # define DECLARE_RETURN_VALUE_USED int return_value_used = 1;
 # define EXCEPTION_P(_ex, _zp) ZVAL_OBJ(&_zp, _ex)
-# define PHONGO_STREAM_ID(stream) stream->res ? stream->res->handle : -1
-# define ADD_ASSOC_STRING(_zv, _key, _value) add_assoc_string_ex(_zv, ZEND_STRL(_key), _value);
-# define ADD_ASSOC_STRINGL(_zv, _key, _value, _len) add_assoc_stringl_ex(_zv, ZEND_STRL(_key), _value, _len);
+# define ADD_ASSOC_STRING(_zv, _key, _value) add_assoc_string_ex(_zv, ZEND_STRL(_key), (char *)(_value));
+# define ADD_ASSOC_STRINGL(_zv, _key, _value, _len) add_assoc_stringl_ex(_zv, ZEND_STRL(_key), (char *)(_value), _len);
+# define ADD_ASSOC_STRING_EX(_zv, _key, _key_len, _value, _value_len) add_assoc_stringl_ex(_zv, _key, _key_len, (char *)(_value), _value_len);
 # define ADD_ASSOC_LONG_EX(_zv, _key, _value) add_assoc_long_ex(_zv, ZEND_STRL(_key), _value);
-# define ADD_ASSOC_STRING_EX(_zv, _key, _key_len, _value, _value_len) add_assoc_stringl_ex(_zv, _key, _key_len, _value, _value_len);
 # define ADD_ASSOC_ZVAL_EX(_zv, _key, _value) add_assoc_zval_ex(_zv, ZEND_STRL(_key), _value);
 # define ADD_ASSOC_ZVAL(_zv, _key, _value) add_assoc_zval(_zv, _key, _value);
 # define ADD_ASSOC_NULL_EX(_zv, _key) add_assoc_null_ex(_zv, ZEND_STRL(_key));
@@ -138,40 +108,40 @@
 # define ZEND_HASH_APPLY_COUNT(ht) (ht)->u.v.nApplyCount
 # define PHONGO_RETVAL_STRINGL(s, slen) RETVAL_STRINGL(s, slen)
 # define PHONGO_RETURN_STRINGL(s, slen) RETURN_STRINGL(s, slen)
+# define PHONGO_RETVAL_STRING(s) RETVAL_STRING(s)
 # define PHONGO_RETURN_STRING(s) RETURN_STRING(s)
+# define PHONGO_RETVAL_SMART_STR(val) PHONGO_RETVAL_STRINGL(ZSTR_VAL((val).s), ZSTR_LEN((val).s));
 #else
 # define phongo_char char
-# define phongo_char_pdup(str) pestrdup(filename, 1)
-# define phongo_char_free(str) _efree(str ZEND_FILE_LINE_CC ZEND_FILE_LINE_CC)
 # define phongo_long long
 # define PHONGO_LONG_FORMAT "ld"
 # define SIZEOF_PHONGO_LONG SIZEOF_LONG
-# define phongo_str(str) str
+# define ZSTR_VAL(str) str
 # define phongo_create_object_retval zend_object_value
 # define PHONGO_ALLOC_OBJECT_T(_obj_t, _class_type) (_obj_t *)ecalloc(1, sizeof(_obj_t))
 # define PHONGO_TSRMLS_FETCH_FROM_CTX(user_data) TSRMLS_FETCH_FROM_CTX(user_data)
 # define SUPPRESS_UNUSED_WARNING(x) (void)x;
 # define DECLARE_RETURN_VALUE_USED
 # define EXCEPTION_P(_ex, _zp) _zp = _ex
-# define PHONGO_STREAM_ID(stream) stream->rsrc_id
-# define ADD_ASSOC_STRING(_zv, _key, _value) add_assoc_string_ex(_zv, ZEND_STRS(_key), _value, 1);
-# define ADD_ASSOC_STRINGL(_zv, _key, _value, _len) add_assoc_stringl_ex(_zv, ZEND_STRS(_key), _value, _len, 1);
-# define ADD_ASSOC_STRING_EX(_zv, _key, _key_len, _value, _value_len) add_assoc_stringl_ex(_zv, _key, _key_len+1, _value, _value_len, 1);
+# define ADD_ASSOC_STRING(_zv, _key, _value) add_assoc_string_ex(_zv, ZEND_STRS(_key), (char *)(_value), 1);
+# define ADD_ASSOC_STRINGL(_zv, _key, _value, _len) add_assoc_stringl_ex(_zv, ZEND_STRS(_key), (char *)(_value), _len, 1);
+# define ADD_ASSOC_STRING_EX(_zv, _key, _key_len, _value, _value_len) add_assoc_stringl_ex(_zv, _key, _key_len+1, (char *)(_value), _value_len, 1);
 # define ADD_ASSOC_LONG_EX(_zv, _key, _value) add_assoc_long_ex(_zv, ZEND_STRS(_key), _value);
 # define ADD_ASSOC_ZVAL_EX(_zv, _key, _value) add_assoc_zval_ex(_zv, ZEND_STRS(_key), _value);
 # define ADD_ASSOC_ZVAL(_zv, _key, _value) add_assoc_zval(_zv, _key, _value);
 # define ADD_ASSOC_NULL_EX(_zv, _key) add_assoc_null_ex(_zv, ZEND_STRS(_key));
 # define ADD_ASSOC_BOOL_EX(_zv, _key, _value) add_assoc_bool_ex(_zv, ZEND_STRS(_key), _value);
 # define ADD_INDEX_STRINGL(_zv, _ind, _value, _len) add_index_stringl(_zv, _ind, _value, _len, 0);
-# define Z_PHPDATE_P(object) zend_object_store_get_object(object TSRMLS_CC)
+# define Z_PHPDATE_P(object) ((php_date_obj*)zend_object_store_get_object(object TSRMLS_CC))
 # define Z_ISUNDEF(x) !x
 # define phongo_free_object_arg void
 # define phongo_zpp_char_len int
 # define ZEND_HASH_APPLY_COUNT(ht) (ht)->nApplyCount
 # define PHONGO_RETVAL_STRINGL(s, slen) RETVAL_STRINGL(s, slen, 1)
 # define PHONGO_RETURN_STRINGL(s, slen) RETURN_STRINGL(s, slen, 1)
+# define PHONGO_RETVAL_STRING(s) RETVAL_STRING(s, 1)
 # define PHONGO_RETURN_STRING(s) RETURN_STRING(s, 1)
-# define PHP_STREAM_CONTEXT(stream) ((php_stream_context*) (stream)->context)
+# define PHONGO_RETVAL_SMART_STR(val) PHONGO_RETVAL_STRINGL((val).c, (val).len);
 #endif
 
 #if SIZEOF_PHONGO_LONG == 8
@@ -180,23 +150,13 @@
 #elif SIZEOF_PHONGO_LONG == 4
 # define ADD_INDEX_INT64(zval, index, value) \
     if (value > INT32_MAX || value < INT32_MIN) { \
-        char *tmp; \
-        int tmp_len; \
-        mongoc_log(MONGOC_LOG_LEVEL_WARNING, MONGOC_LOG_DOMAIN, "Integer overflow detected on your platform: %lld", value); \
-        tmp_len = spprintf(&tmp, 0, "%lld", value); \
-        ADD_INDEX_STRINGL(zval, index, tmp, tmp_len); \
-        efree(tmp); \
+        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", value); \
     } else { \
         add_index_long(zval, index, val); \
     }
 # define ADD_ASSOC_INT64(zval, key, value) \
     if (value > INT32_MAX || value < INT32_MIN) { \
-        char *tmp; \
-        int tmp_len; \
-        mongoc_log(MONGOC_LOG_LEVEL_WARNING, MONGOC_LOG_DOMAIN, "Integer overflow detected on your platform: %lld", value); \
-        tmp_len = spprintf(&tmp, 0, "%lld", value); \
-        ADD_ASSOC_STRING_EX(zval, key, strlen(key), tmp, tmp_len); \
-        efree(tmp); \
+        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", value); \
     } else { \
         add_assoc_long(zval, key, value); \
     }
@@ -204,7 +164,6 @@
 # error Unsupported architecture (integers are neither 32-bit nor 64-bit)
 #endif
 
-void *x509_from_zval(zval *zval TSRMLS_DC);
 void phongo_add_exception_prop(const char *prop, int prop_len, zval *value TSRMLS_DC);
 
 
