@@ -22,13 +22,12 @@
 /* External libs */
 #include "bson.h"
 #include "mongoc.h"
-#include "mongoc-topology-description-private.h"
 
 #define phpext_mongodb_ptr &mongodb_module_entry
 extern zend_module_entry mongodb_module_entry;
 
 /* FIXME: Its annoying to bump version. Move into phongo_version.h.in */
-#define MONGODB_VERSION_S "1.1.7"
+#define MONGODB_VERSION_S "1.2.0"
 #define MONGODB_STABILITY_S "stable"
 #define MONGODB_VERSION   MONGODB_VERSION_S
 
@@ -41,17 +40,11 @@ extern zend_module_entry mongodb_module_entry;
 #	define PHONGO_API
 #endif
 
-#ifdef ZTS
-#	include "TSRM.h"
-#	define PHONGO_STREAM_CTX(x) x
-#else
-#	define PHONGO_STREAM_CTX(x) NULL
-#endif
-
 ZEND_BEGIN_MODULE_GLOBALS(mongodb)
 	char *debug;
 	FILE *debug_fd;
 	bson_mem_vtable_t bsonMemVTable;
+	HashTable clients;
 ZEND_END_MODULE_GLOBALS(mongodb)
 
 #if PHP_VERSION_ID >= 70000
@@ -70,6 +63,29 @@ ZEND_END_MODULE_GLOBALS(mongodb)
 
 #include "php_phongo_classes.h"
 
+/* This enum is necessary since mongoc_server_description_type_t is private and
+ * we need to translate strings returned by mongoc_server_description_type() to
+ * Server integer constants. */
+typedef enum {
+	PHONGO_SERVER_UNKNOWN           = 0,
+	PHONGO_SERVER_STANDALONE        = 1,
+	PHONGO_SERVER_MONGOS            = 2,
+	PHONGO_SERVER_POSSIBLE_PRIMARY  = 3,
+	PHONGO_SERVER_RS_PRIMARY        = 4,
+	PHONGO_SERVER_RS_SECONDARY      = 5,
+	PHONGO_SERVER_RS_ARBITER        = 6,
+	PHONGO_SERVER_RS_OTHER          = 7,
+	PHONGO_SERVER_RS_GHOST          = 8,
+	PHONGO_SERVER_DESCRIPTION_TYPES = 9,
+} php_phongo_server_description_type_t;
+
+typedef struct {
+	php_phongo_server_description_type_t  type;
+	const char                           *name;
+} php_phongo_server_description_type_map_t;
+
+extern php_phongo_server_description_type_map_t php_phongo_server_description_type_map[];
+
 typedef enum {
 	PHONGO_ERROR_INVALID_ARGUMENT    = 1,
 	PHONGO_ERROR_RUNTIME             = 2,
@@ -80,25 +96,8 @@ typedef enum {
 	PHONGO_ERROR_LOGIC               = 9
 } php_phongo_error_domain_t;
 
-typedef struct
-{
-	void (*writer)(mongoc_stream_t *stream, int32_t timeout_msec, ssize_t sent, size_t iovcnt);
-} php_phongo_stream_logger;
-
-typedef struct
-{
-	mongoc_stream_t           vtable;
-	php_stream               *stream;
-	const mongoc_uri_t       *uri;
-	const mongoc_host_list_t *host;
-#if ZTS
-	void ***tsrm_ls;
-#endif
-} php_phongo_stream_socket;
-
-
-PHONGO_API zend_class_entry* phongo_exception_from_mongoc_domain(uint32_t /* mongoc_error_domain_t */ domain, uint32_t /* mongoc_error_code_t */ code);
-PHONGO_API zend_class_entry* phongo_exception_from_phongo_domain(php_phongo_error_domain_t domain);
+zend_class_entry* phongo_exception_from_mongoc_domain(uint32_t /* mongoc_error_domain_t */ domain, uint32_t /* mongoc_error_code_t */ code);
+zend_class_entry* phongo_exception_from_phongo_domain(php_phongo_error_domain_t domain);
 void phongo_throw_exception(php_phongo_error_domain_t domain TSRMLS_DC, const char *format, ...)
 #if PHP_VERSION_ID < 70000
 # ifndef PHP_WIN32
@@ -112,39 +111,41 @@ void phongo_throw_exception(php_phongo_error_domain_t domain TSRMLS_DC, const ch
 ;
 void phongo_throw_exception_from_bson_error_t(bson_error_t *error TSRMLS_DC);
 
-PHONGO_API zend_object_handlers *phongo_get_std_object_handlers(void);
+zend_object_handlers *phongo_get_std_object_handlers(void);
 
 void                     phongo_server_init          (zval *return_value, zval *manager, int server_id TSRMLS_DC);
 void                     phongo_readconcern_init     (zval *return_value, const mongoc_read_concern_t *read_concern TSRMLS_DC);
 void                     phongo_readpreference_init  (zval *return_value, const mongoc_read_prefs_t *read_prefs TSRMLS_DC);
 void                     phongo_writeconcern_init    (zval *return_value, const mongoc_write_concern_t *write_concern TSRMLS_DC);
-bool                     phongo_query_init           (php_phongo_query_t *query, bson_t *filter, bson_t *options TSRMLS_DC);
 mongoc_bulk_operation_t* phongo_bulkwrite_init       (zend_bool ordered);
-bool                     phongo_execute_write        (zval *manager, const char *namespace, mongoc_bulk_operation_t *bulk, const mongoc_write_concern_t *write_concern, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
-int                      phongo_execute_command      (zval *manager, const char *db,        const bson_t *command,           const mongoc_read_prefs_t *read_preference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
-int                      phongo_execute_query        (zval *manager, const char *namespace, const php_phongo_query_t *query, const mongoc_read_prefs_t *read_preference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
+bool                     phongo_execute_write        (zval *manager, const char *namespace, php_phongo_bulkwrite_t  *bulk_write, const mongoc_write_concern_t *write_concern, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
+int                      phongo_execute_command      (zval *manager, const char *db, zval *zcommand, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
+int                      phongo_execute_query        (zval *manager, const char *namespace, zval *zquery, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
 
-mongoc_stream_t*         phongo_stream_initiator     (const mongoc_uri_t *uri, const mongoc_host_list_t *host, void *user_data, bson_error_t *error);
 const mongoc_read_concern_t*  phongo_read_concern_from_zval   (zval *zread_concern TSRMLS_DC);
 const mongoc_read_prefs_t*    phongo_read_preference_from_zval(zval *zread_preference TSRMLS_DC);
 const mongoc_write_concern_t* phongo_write_concern_from_zval  (zval *zwrite_concern TSRMLS_DC);
-const php_phongo_query_t*     phongo_query_from_zval          (zval *zquery TSRMLS_DC);
 
-void php_phongo_server_to_zval(zval *retval, const mongoc_server_description_t *sd);
+php_phongo_server_description_type_t php_phongo_server_description_type(mongoc_server_description_t *sd);
+
+void php_phongo_read_preference_prep_tagsets(zval *tagSets TSRMLS_DC);
+bool php_phongo_read_preference_tags_are_valid(const bson_t *tags);
+
+void php_phongo_server_to_zval(zval *retval, mongoc_server_description_t *sd);
 void php_phongo_read_concern_to_zval(zval *retval, const mongoc_read_concern_t *read_concern);
 void php_phongo_read_preference_to_zval(zval *retval, const mongoc_read_prefs_t *read_prefs);
 void php_phongo_write_concern_to_zval(zval *retval, const mongoc_write_concern_t *write_concern);
 void php_phongo_cursor_to_zval(zval *retval, const mongoc_cursor_t *cursor);
 
-bool phongo_manager_init(php_phongo_manager_t *manager, const char *uri_string, bson_t *bson_options, zval *driverOptions TSRMLS_DC);
+void phongo_manager_init(php_phongo_manager_t *manager, const char *uri_string, zval *options, zval *driverOptions TSRMLS_DC);
 void php_phongo_objectid_new_from_oid(zval *object, const bson_oid_t *oid TSRMLS_DC);
 void php_phongo_cursor_id_new_from_id(zval *object, int64_t cursorid TSRMLS_DC);
 void php_phongo_new_utcdatetime_from_epoch(zval *object, int64_t msec_since_epoch TSRMLS_DC);
-void php_phongo_new_datetime_from_utcdatetime(zval *object, int64_t milliseconds TSRMLS_DC);
 void php_phongo_new_timestamp_from_increment_and_timestamp(zval *object, uint32_t increment, uint32_t timestamp TSRMLS_DC);
 void php_phongo_new_javascript_from_javascript(int init, zval *object, const char *code, size_t code_len TSRMLS_DC);
 void php_phongo_new_javascript_from_javascript_and_scope(int init, zval *object, const char *code, size_t code_len, const bson_t *scope TSRMLS_DC);
 void php_phongo_new_binary_from_binary_and_type(zval *object, const char *data, size_t data_len, bson_subtype_t type TSRMLS_DC);
+void php_phongo_new_decimal128(zval *object, const bson_decimal128_t *decimal TSRMLS_DC);
 void php_phongo_new_regex_from_regex_and_options(zval *object, const char *pattern, const char *flags TSRMLS_DC);
 
 zend_bool phongo_writeerror_init(zval *return_value, bson_t *bson TSRMLS_DC);
@@ -154,19 +155,19 @@ void php_phongo_cursor_free(php_phongo_cursor_t *cursor);
 zend_object_iterator* php_phongo_cursor_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC);
 
 #if PHP_VERSION_ID >= 70000
-#define PHONGO_CE_INIT(ce) do {                     \
-	ce->ce_flags    |= ZEND_ACC_FINAL;              \
-	ce->serialize    = zend_class_serialize_deny;   \
-	ce->unserialize  = zend_class_unserialize_deny; \
+#define PHONGO_CE_FINAL(ce) do {    \
+	ce->ce_flags |= ZEND_ACC_FINAL; \
 } while(0);
 #else
-#define PHONGO_CE_INIT(ce) do {                     \
-	ce->ce_flags    |= ZEND_ACC_FINAL_CLASS;        \
-	ce->serialize    = zend_class_serialize_deny;   \
-	ce->unserialize  = zend_class_unserialize_deny; \
+#define PHONGO_CE_FINAL(ce) do {          \
+	ce->ce_flags |= ZEND_ACC_FINAL_CLASS; \
 } while(0);
 #endif
 
+#define PHONGO_CE_DISABLE_SERIALIZATION(ce) do {   \
+    ce->serialize   = zend_class_serialize_deny;   \
+    ce->unserialize = zend_class_unserialize_deny; \
+} while(0);
 
 #ifdef PHP_DEBUG
 void _phongo_debug_bson(bson_t *bson);
