@@ -27,8 +27,6 @@
 /* External libs */
 #include <bson.h>
 #include <mongoc.h>
-#include <mongoc-cursor-private.h>
-#include <mongoc-bulk-operation-private.h>
 
 /* PHP Core stuff */
 #include <php.h>
@@ -187,29 +185,19 @@ PHP_METHOD(Cursor, isDead)
 
 ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_setTypeMap, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, typemap, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_toArray, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_getId, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_getServer, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_isDead, 0, 0, 0)
-ZEND_END_ARG_INFO();
+ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_void, 0, 0, 0)
+ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_cursor_me[] = {
-	PHP_ME(Server, __construct, NULL, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
+	PHP_ME(Server, __construct, ai_Cursor_void, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
 	PHP_ME(Cursor, setTypeMap, ai_Cursor_setTypeMap, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, toArray, ai_Cursor_toArray, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, getId, ai_Cursor_getId, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, getServer, ai_Cursor_getServer, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, isDead, ai_Cursor_isDead, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Manager, __wakeUp, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, toArray, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Cursor, getId, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Cursor, getServer, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Cursor, isDead, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Manager, __wakeup, ai_Cursor_void, ZEND_ACC_PUBLIC)
 	PHP_FE_END
 };
 
@@ -223,6 +211,26 @@ static void php_phongo_cursor_free_object(phongo_free_object_arg *object TSRMLS_
 
 	zend_object_std_dtor(&intern->std TSRMLS_CC);
 
+	if (intern->database) {
+		efree(intern->database);
+	}
+
+	if (intern->collection) {
+		efree(intern->collection);
+	}
+
+	if (!Z_ISUNDEF(intern->query)) {
+		zval_ptr_dtor(&intern->query);
+	}
+
+	if (!Z_ISUNDEF(intern->command)) {
+		zval_ptr_dtor(&intern->command);
+	}
+
+	if (!Z_ISUNDEF(intern->read_preference)) {
+		zval_ptr_dtor(&intern->read_preference);
+	}
+
 	php_phongo_cursor_free(intern);
 
 	zval_ptr_dtor(&intern->manager);
@@ -269,26 +277,86 @@ HashTable *php_phongo_cursor_get_debug_info(zval *object, int *is_temp TSRMLS_DC
 	*is_temp = 1;
 	intern = Z_CURSOR_OBJ_P(object);
 
-	array_init_size(&retval, 2);
-	if (intern->cursor) {
-#if PHP_VERSION_ID >= 70000
-		zval zcursor;
+	array_init_size(&retval, 9);
 
-		php_phongo_cursor_to_zval(&zcursor, intern->cursor);
-		ADD_ASSOC_ZVAL_EX(&retval, "cursor", &zcursor);
-#else
-		zval *zcursor = NULL;
-
-		MAKE_STD_ZVAL(zcursor);
-
-		php_phongo_cursor_to_zval(zcursor, intern->cursor);
-		ADD_ASSOC_ZVAL_EX(&retval, "cursor", zcursor);
-#endif
+	if (intern->database) {
+		ADD_ASSOC_STRING(&retval, "database", intern->database);
 	} else {
-		ADD_ASSOC_NULL_EX(&retval, "cursor");
+		ADD_ASSOC_NULL_EX(&retval, "database");
 	}
 
-	ADD_ASSOC_LONG_EX(&retval, "server_id", intern->server_id);
+	if (intern->collection) {
+		ADD_ASSOC_STRING(&retval, "collection", intern->collection);
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "collection");
+	}
+
+	if (!Z_ISUNDEF(intern->query)) {
+#if PHP_VERSION_ID >= 70000
+		ADD_ASSOC_ZVAL_EX(&retval, "query", &intern->query);
+		Z_ADDREF(intern->query);
+#else
+		ADD_ASSOC_ZVAL_EX(&retval, "query", intern->query);
+		Z_ADDREF_P(intern->query);
+#endif
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "query");
+	}
+
+	if (!Z_ISUNDEF(intern->command)) {
+#if PHP_VERSION_ID >= 70000
+		ADD_ASSOC_ZVAL_EX(&retval, "command", &intern->command);
+		Z_ADDREF(intern->command);
+#else
+		ADD_ASSOC_ZVAL_EX(&retval, "command", intern->command);
+		Z_ADDREF_P(intern->command);
+#endif
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "command");
+	}
+
+	if (!Z_ISUNDEF(intern->read_preference)) {
+#if PHP_VERSION_ID >= 70000
+		ADD_ASSOC_ZVAL_EX(&retval, "readPreference", &intern->read_preference);
+		Z_ADDREF(intern->read_preference);
+#else
+		ADD_ASSOC_ZVAL_EX(&retval, "readPreference", intern->read_preference);
+		Z_ADDREF_P(intern->read_preference);
+#endif
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "readPreference");
+	}
+
+	ADD_ASSOC_BOOL_EX(&retval, "isDead", !mongoc_cursor_is_alive(intern->cursor));
+
+	ADD_ASSOC_LONG_EX(&retval, "currentIndex", intern->current);
+
+	if (!Z_ISUNDEF(intern->visitor_data.zchild)) {
+#if PHP_VERSION_ID >= 70000
+		ADD_ASSOC_ZVAL_EX(&retval, "currentDocument", &intern->visitor_data.zchild);
+		/*Z_ADDREF(intern->visitor_data.zchild);*/
+#else
+		ADD_ASSOC_ZVAL_EX(&retval, "currentDocument", intern->visitor_data.zchild);
+		Z_ADDREF_P(intern->visitor_data.zchild);
+#endif
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "currentDocument");
+	}
+
+	{
+#if PHP_VERSION_ID >= 70000
+		zval server;
+
+		phongo_server_init(&server, &intern->manager, intern->server_id TSRMLS_CC);
+		ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
+#else
+		zval *server = NULL;
+
+		MAKE_STD_ZVAL(server);
+		phongo_server_init(server, intern->manager, intern->server_id TSRMLS_CC);
+		ADD_ASSOC_ZVAL_EX(&retval, "server", server);
+#endif
+	}
 
 	return Z_ARRVAL(retval);
 
@@ -304,7 +372,8 @@ PHP_MINIT_FUNCTION(Cursor)
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Cursor", php_phongo_cursor_me);
 	php_phongo_cursor_ce = zend_register_internal_class(&ce TSRMLS_CC);
 	php_phongo_cursor_ce->create_object = php_phongo_cursor_create_object;
-	PHONGO_CE_INIT(php_phongo_cursor_ce);
+	PHONGO_CE_FINAL(php_phongo_cursor_ce);
+	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_cursor_ce);
 	php_phongo_cursor_ce->get_iterator = php_phongo_cursor_get_iterator;
 
 	memcpy(&php_phongo_handler_cursor, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
