@@ -27,7 +27,6 @@
 /* External libs */
 #include <bson.h>
 #include <mongoc.h>
-#include <mongoc-client-private.h>
 
 /* PHP Core stuff */
 #include <php.h>
@@ -49,8 +48,8 @@ PHONGO_API zend_class_entry *php_phongo_server_ce;
 zend_object_handlers php_phongo_handler_server;
 
 
-/* {{{ proto MongoDB\Driver\Server Server::__construct()
-   Throws exception -- can only be created internally */
+/* {{{ proto void Server::__construct()
+   Throws MongoDB\Driver\RuntimeException (can only be created internally) */
 PHP_METHOD(Server, __construct)
 {
 	SUPPRESS_UNUSED_WARNING(return_value) SUPPRESS_UNUSED_WARNING(return_value_used) SUPPRESS_UNUSED_WARNING(return_value_ptr) (void)ZEND_NUM_ARGS(); (void)getThis();
@@ -58,8 +57,9 @@ PHP_METHOD(Server, __construct)
 	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Accessing private constructor");
 }
 /* }}} */
+
 /* {{{ proto MongoDB\Driver\Cursor Server::executeCommand(string $db, MongoDB\Driver\Command $command[, MongoDB\Driver\ReadPreference $readPreference = null]))
-   Executes a command on this server */
+   Executes a Command on this Server */
 PHP_METHOD(Server, executeCommand)
 {
 	php_phongo_server_t      *intern;
@@ -67,7 +67,6 @@ PHP_METHOD(Server, executeCommand)
 	phongo_zpp_char_len       db_len;
 	zval                     *command;
 	zval                     *readPreference = NULL;
-	php_phongo_command_t     *cmd;
 	DECLARE_RETURN_VALUE_USED
 	SUPPRESS_UNUSED_WARNING(return_value_ptr)
 
@@ -78,23 +77,22 @@ PHP_METHOD(Server, executeCommand)
 		return;
 	}
 
-
-	cmd = Z_COMMAND_OBJ_P(command);
 #if PHP_VERSION_ID >= 70000
-	phongo_execute_command(&intern->manager, db, cmd->bson, phongo_read_preference_from_zval(readPreference TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
+	phongo_execute_command(&intern->manager, db, command, readPreference, intern->server_id, return_value, return_value_used TSRMLS_CC);
 #else
-	phongo_execute_command(intern->manager, db, cmd->bson, phongo_read_preference_from_zval(readPreference TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
+	phongo_execute_command(intern->manager, db, command, readPreference, intern->server_id, return_value, return_value_used TSRMLS_CC);
 #endif
 }
 /* }}} */
-/* {{{ proto MongoDB\Driver\Cursor Server::executeQuery(string $namespace, MongoDB\Driver\Query $zquery[, MongoDB\Driver\ReadPreference $readPreference = null]))
-   Executes a Query */
+
+/* {{{ proto MongoDB\Driver\Cursor Server::executeQuery(string $namespace, MongoDB\Driver\Query $query[, MongoDB\Driver\ReadPreference $readPreference = null]))
+   Executes a Query on this Server */
 PHP_METHOD(Server, executeQuery)
 {
 	php_phongo_server_t      *intern;
 	char                     *namespace;
 	phongo_zpp_char_len       namespace_len;
-	zval                     *zquery;
+	zval                     *query;
 	zval                     *readPreference = NULL;
 	DECLARE_RETURN_VALUE_USED
 	SUPPRESS_UNUSED_WARNING(return_value_ptr)
@@ -102,19 +100,21 @@ PHP_METHOD(Server, executeQuery)
 
 	intern = Z_SERVER_OBJ_P(getThis());
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sO|O!", &namespace, &namespace_len, &zquery, php_phongo_query_ce, &readPreference, php_phongo_readpreference_ce) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sO|O!", &namespace, &namespace_len, &query, php_phongo_query_ce, &readPreference, php_phongo_readpreference_ce) == FAILURE) {
 		return;
 	}
 
 #if PHP_VERSION_ID >= 70000
-	phongo_execute_query(&intern->manager, namespace, phongo_query_from_zval(zquery TSRMLS_CC), phongo_read_preference_from_zval(readPreference TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
+	phongo_execute_query(&intern->manager, namespace, query, readPreference, intern->server_id, return_value, return_value_used TSRMLS_CC);
 #else
-	phongo_execute_query(intern->manager, namespace, phongo_query_from_zval(zquery TSRMLS_CC), phongo_read_preference_from_zval(readPreference TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
+	phongo_execute_query(intern->manager, namespace, query, readPreference, intern->server_id, return_value, return_value_used TSRMLS_CC);
 #endif
 }
 /* }}} */
+
 /* {{{ proto MongoDB\Driver\WriteResult Server::executeBulkWrite(string $namespace, MongoDB\Driver\BulkWrite $zbulk[, MongoDB\Driver\WriteConcern $writeConcern = null])
-   Executes a write operation bulk (e.g. insert, update, delete) */
+   Executes a BulkWrite (i.e. any number of insert, update, and delete ops) on
+   this Server */
 PHP_METHOD(Server, executeBulkWrite)
 {
 	php_phongo_server_t      *intern;
@@ -136,19 +136,19 @@ PHP_METHOD(Server, executeBulkWrite)
 
 	bulk = Z_BULKWRITE_OBJ_P(zbulk);
 #if PHP_VERSION_ID >= 70000
-	phongo_execute_write(&intern->manager, namespace, bulk->bulk, phongo_write_concern_from_zval(zwrite_concern TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
+	phongo_execute_write(&intern->manager, namespace, bulk, phongo_write_concern_from_zval(zwrite_concern TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
 #else
-	phongo_execute_write(intern->manager, namespace, bulk->bulk, phongo_write_concern_from_zval(zwrite_concern TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
+	phongo_execute_write(intern->manager, namespace, bulk, phongo_write_concern_from_zval(zwrite_concern TSRMLS_CC), intern->server_id, return_value, return_value_used TSRMLS_CC);
 #endif
 }
 /* }}} */
+
 /* {{{ proto string Server::getHost()
-   Returns the hostname used to connect to this Server */
+   Returns the hostname for this Server */
 PHP_METHOD(Server, getHost)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 	intern = Z_SERVER_OBJ_P(getThis());
@@ -157,20 +157,22 @@ PHP_METHOD(Server, getHost)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		PHONGO_RETURN_STRING(sd->host.host);
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		PHONGO_RETVAL_STRING(mongoc_server_description_host(sd)->host);
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto array Server::getTags()
-   Returns the currently configured tags for this node */
+   Returns the currently configured tags for this Server */
 PHP_METHOD(Server, getTags)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -181,29 +183,44 @@ PHP_METHOD(Server, getTags)
 	}
 
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		php_phongo_bson_state  state = PHONGO_BSON_STATE_INITIALIZER;
-		state.map.root_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
-		state.map.document_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		const bson_t *is_master = mongoc_server_description_ismaster(sd);
+		bson_iter_t   iter;
+
+		if (bson_iter_init_find(&iter, is_master, "tags") && BSON_ITER_HOLDS_DOCUMENT(&iter)) {
+			const uint8_t         *bytes;
+			uint32_t               len;
+			php_phongo_bson_state  state = PHONGO_BSON_STATE_INITIALIZER;
+
+			state.map.root_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
+			state.map.document_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
+
+			bson_iter_document(&iter, &len, &bytes);
+			phongo_bson_to_zval_ex(bytes, len, &state);
+			mongoc_server_description_destroy(sd);
 
-		phongo_bson_to_zval_ex(bson_get_data(&sd->tags), sd->tags.len, &state);
 #if PHP_VERSION_ID >= 70000
-		RETURN_ZVAL(&state.zchild, 0, 1);
+			RETURN_ZVAL(&state.zchild, 0, 1);
 #else
-		RETURN_ZVAL(state.zchild, 0, 1);
+			RETURN_ZVAL(state.zchild, 0, 1);
 #endif
+		}
+
+		array_init(return_value);
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto array Server::getInfo()
-   Returns the last isMaster() result document */
+   Returns the last isMaster result document for this Server */
 PHP_METHOD(Server, getInfo)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -214,12 +231,15 @@ PHP_METHOD(Server, getInfo)
 	}
 
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		const bson_t          *is_master = mongoc_server_description_ismaster(sd);
 		php_phongo_bson_state  state = PHONGO_BSON_STATE_INITIALIZER;
+
 		state.map.root_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
 		state.map.document_type = PHONGO_TYPEMAP_NATIVE_ARRAY;
 
-		phongo_bson_to_zval_ex(bson_get_data(&sd->last_is_master), sd->last_is_master.len, &state);
+		phongo_bson_to_zval_ex(bson_get_data(is_master), is_master->len, &state);
+		mongoc_server_description_destroy(sd);
 
 #if PHP_VERSION_ID >= 70000
 		RETURN_ZVAL(&state.zchild, 0, 1);
@@ -228,16 +248,16 @@ PHP_METHOD(Server, getInfo)
 #endif
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto integer Server::getLatency()
-   Returns the last messured latency */
+   Returns the last messured latency for this Server */
 PHP_METHOD(Server, getLatency)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -247,20 +267,22 @@ PHP_METHOD(Server, getLatency)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		RETURN_LONG(sd->round_trip_time);
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		RETVAL_LONG((phongo_long) mongoc_server_description_round_trip_time(sd));
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto integer Server::getPort()
-   Returns the port used to create this Server */
+   Returns the port for this Server */
 PHP_METHOD(Server, getPort)
 {
 	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -270,20 +292,22 @@ PHP_METHOD(Server, getPort)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		RETURN_LONG(sd->host.port);
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		RETVAL_LONG(mongoc_server_description_host(sd)->port);
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto integer Server::getType()
    Returns the node type of this Server */
 PHP_METHOD(Server, getType)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -293,20 +317,22 @@ PHP_METHOD(Server, getType)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		RETURN_LONG(sd->type);
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		RETVAL_LONG(php_phongo_server_description_type(sd));
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto bool Server::isPrimary()
-   Checks if this is a special "Primary" member of a RepilcaSet */
+   Returns whether this Server is a primary member of a replica set */
 PHP_METHOD(Server, isPrimary)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -316,20 +342,22 @@ PHP_METHOD(Server, isPrimary)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		RETURN_BOOL(sd->type == MONGOC_SERVER_RS_PRIMARY);
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		RETVAL_BOOL(!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_PRIMARY].name));
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto bool Server::isSecondary()
-   Checks if this is a special "Secondary" member of a RepilcaSet */
+   Returns whether this Server is a secondary member of a replica set */
 PHP_METHOD(Server, isSecondary)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -339,20 +367,22 @@ PHP_METHOD(Server, isSecondary)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		RETURN_BOOL(sd->type == MONGOC_SERVER_RS_SECONDARY);
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		RETVAL_BOOL(!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_SECONDARY].name));
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto bool Server::isArbiter()
-   Checks if this is a special "Arbiter" member of a RepilcaSet */
+   Returns whether this Server is an arbiter member of a replica set */
 PHP_METHOD(Server, isArbiter)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -362,20 +392,22 @@ PHP_METHOD(Server, isArbiter)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		RETURN_BOOL(sd->type == MONGOC_SERVER_RS_ARBITER);
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		RETVAL_BOOL(!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_ARBITER].name));
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto bool Server::isHidden()
-   Checks if this is a special "hidden" member of a RepilcaSet */
+   Returns whether this Server is a hidden member of a replica set */
 PHP_METHOD(Server, isHidden)
 {
-	php_phongo_server_t      *intern;
+	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -385,22 +417,24 @@ PHP_METHOD(Server, isHidden)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
 		bson_iter_t iter;
 
-		RETURN_BOOL(bson_iter_init_find_case(&iter, &sd->last_is_master, "hidden") && bson_iter_as_bool(&iter));
+		RETVAL_BOOL(bson_iter_init_find_case(&iter, mongoc_server_description_ismaster(sd), "hidden") && bson_iter_as_bool(&iter));
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
+
 /* {{{ proto bool Server::isPassive()
-   Checks if this is a special passive node member of a ReplicaSet */
+   Returns whether this Server is a passive member of a replica set */
 PHP_METHOD(Server, isPassive)
 {
 	php_phongo_server_t         *intern;
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
 
 
@@ -410,13 +444,15 @@ PHP_METHOD(Server, isPassive)
 		return;
 	}
 
-	if ((sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
+	if ((sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
 		bson_iter_t iter;
 
-		RETURN_BOOL(bson_iter_init_find_case(&iter, &sd->last_is_master, "passive") && bson_iter_as_bool(&iter));
+		RETVAL_BOOL(bson_iter_init_find_case(&iter, mongoc_server_description_ismaster(sd), "passive") && bson_iter_as_bool(&iter));
+		mongoc_server_description_destroy(sd);
+		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 }
 /* }}} */
 
@@ -426,71 +462,40 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Server_executeCommand, 0, 0, 2)
 	ZEND_ARG_INFO(0, db)
 	ZEND_ARG_OBJ_INFO(0, command, MongoDB\\Driver\\Command, 0)
 	ZEND_ARG_OBJ_INFO(0, readPreference, MongoDB\\Driver\\ReadPreference, 1)
-ZEND_END_ARG_INFO();
+ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(ai_Server_executeQuery, 0, 0, 2)
 	ZEND_ARG_INFO(0, namespace)
 	ZEND_ARG_OBJ_INFO(0, zquery, MongoDB\\Driver\\Query, 0)
 	ZEND_ARG_OBJ_INFO(0, readPreference, MongoDB\\Driver\\ReadPreference, 1)
-ZEND_END_ARG_INFO();
+ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(ai_Server_executeBulkWrite, 0, 0, 2)
 	ZEND_ARG_INFO(0, namespace)
 	ZEND_ARG_OBJ_INFO(0, zbulk, MongoDB\\Driver\\BulkWrite, 0)
 	ZEND_ARG_OBJ_INFO(0, writeConcern, MongoDB\\Driver\\WriteConcern, 1)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_getHost, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_getTags, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_getInfo, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_getLatency, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_getPort, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_getType, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_isPrimary, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_isSecondary, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_isArbiter, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_isHidden, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_Server_isPassive, 0, 0, 0)
-ZEND_END_ARG_INFO();
+ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_Server_void, 0, 0, 0)
+ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_server_me[] = {
-	PHP_ME(Server, __construct, NULL, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
+	PHP_ME(Server, __construct, ai_Server_void, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
 	PHP_ME(Server, executeCommand, ai_Server_executeCommand, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(Server, executeQuery, ai_Server_executeQuery, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(Server, executeBulkWrite, ai_Server_executeBulkWrite, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, getHost, ai_Server_getHost, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, getTags, ai_Server_getTags, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, getInfo, ai_Server_getInfo, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, getLatency, ai_Server_getLatency, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, getPort, ai_Server_getPort, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, getType, ai_Server_getType, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, isPrimary, ai_Server_isPrimary, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, isSecondary, ai_Server_isSecondary, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, isArbiter, ai_Server_isArbiter, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, isHidden, ai_Server_isHidden, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Server, isPassive, ai_Server_isPassive, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Manager, __wakeUp, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(Server, getHost, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, getTags, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, getInfo, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, getLatency, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, getPort, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, getType, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, isPrimary, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, isSecondary, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, isArbiter, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, isHidden, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Server, isPassive, ai_Server_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Manager, __wakeup, ai_Server_void, ZEND_ACC_PUBLIC)
 	PHP_FE_END
 };
 
@@ -500,30 +505,31 @@ static zend_function_entry php_phongo_server_me[] = {
 /* {{{ Other functions */
 static int php_phongo_server_compare_objects(zval *o1, zval *o2 TSRMLS_DC) /* {{{ */
 {
-    php_phongo_server_t *intern1;
-    php_phongo_server_t *intern2;
-	bson_error_t error1;
-	bson_error_t error2;
+	php_phongo_server_t         *intern1, *intern2;
 	mongoc_server_description_t *sd1, *sd2;
+	int                          retval = 0;
 
-    intern1 = Z_SERVER_OBJ_P(o1);
-    intern2 = Z_SERVER_OBJ_P(o2);
+	intern1 = Z_SERVER_OBJ_P(o1);
+	intern2 = Z_SERVER_OBJ_P(o2);
 
-	sd1 = mongoc_topology_description_server_by_id(&intern1->client->topology->description, intern1->server_id, &error1);
-	sd2 = mongoc_topology_description_server_by_id(&intern2->client->topology->description, intern2->server_id, &error2);
+	sd1 = mongoc_client_get_server_description(intern1->client, intern1->server_id);
+	sd2 = mongoc_client_get_server_description(intern2->client, intern2->server_id);
 
-	if (!sd1 || !sd2) {
-		if (!sd1 && !sd2) {
-			phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server descriptions: %s and %s", error1.message, error2.message);
-		} else if (!sd1) {
-			phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error1.message);
-		} else {
-			phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error2.message);
-		}
-		return 0;
+	if (sd1 && sd2) {
+		retval = strcasecmp(mongoc_server_description_host(sd1)->host_and_port, mongoc_server_description_host(sd2)->host_and_port);
+	} else {
+		phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description(s)");
 	}
 
-	return strcasecmp(sd1->host.host_and_port, sd2->host.host_and_port);
+	if (sd1) {
+		mongoc_server_description_destroy(sd1);
+	}
+
+	if (sd2) {
+		mongoc_server_description_destroy(sd2);
+	}
+
+	return retval;
 } /* }}} */
 /* }}} */
 /* {{{ php_phongo_server_t object handlers */
@@ -573,18 +579,18 @@ HashTable *php_phongo_server_get_debug_info(zval *object, int *is_temp TSRMLS_DC
 	zval                         retval = zval_used_for_init;
 #endif
 	mongoc_server_description_t *sd;
-	bson_error_t                 error;
 
 	*is_temp = 1;
 	intern = Z_SERVER_OBJ_P(object);
 
 
-	if (!(sd = mongoc_topology_description_server_by_id(&intern->client->topology->description, intern->server_id, &error))) {
-		phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description: %s", error.message);
+	if (!(sd = mongoc_client_get_server_description(intern->client, intern->server_id))) {
+		phongo_throw_exception(PHONGO_ERROR_RUNTIME TSRMLS_CC, "Failed to get server description");
 		return NULL;
 	}
 
 	php_phongo_server_to_zval(&retval, sd);
+	mongoc_server_description_destroy(sd);
 
 	return Z_ARRVAL(retval);
 } /* }}} */
@@ -599,7 +605,8 @@ PHP_MINIT_FUNCTION(Server)
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Server", php_phongo_server_me);
 	php_phongo_server_ce = zend_register_internal_class(&ce TSRMLS_CC);
 	php_phongo_server_ce->create_object = php_phongo_server_create_object;
-	PHONGO_CE_INIT(php_phongo_server_ce);
+	PHONGO_CE_FINAL(php_phongo_server_ce);
+	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_server_ce);
 
 	memcpy(&php_phongo_handler_server, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_server.compare_objects = php_phongo_server_compare_objects;
@@ -609,15 +616,15 @@ PHP_MINIT_FUNCTION(Server)
 	php_phongo_handler_server.offset = XtOffsetOf(php_phongo_server_t, std);
 #endif
 
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_UNKNOWN"), MONGOC_SERVER_UNKNOWN TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_STANDALONE"), MONGOC_SERVER_STANDALONE TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_MONGOS"), MONGOC_SERVER_MONGOS TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_POSSIBLE_PRIMARY"), MONGOC_SERVER_POSSIBLE_PRIMARY TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_PRIMARY"), MONGOC_SERVER_RS_PRIMARY TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_SECONDARY"), MONGOC_SERVER_RS_SECONDARY TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_ARBITER"), MONGOC_SERVER_RS_ARBITER TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_OTHER"), MONGOC_SERVER_RS_OTHER TSRMLS_CC);
-	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_GHOST"), MONGOC_SERVER_RS_GHOST TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_UNKNOWN"), PHONGO_SERVER_UNKNOWN TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_STANDALONE"), PHONGO_SERVER_STANDALONE TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_MONGOS"), PHONGO_SERVER_MONGOS TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_POSSIBLE_PRIMARY"), PHONGO_SERVER_POSSIBLE_PRIMARY TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_PRIMARY"), PHONGO_SERVER_RS_PRIMARY TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_SECONDARY"), PHONGO_SERVER_RS_SECONDARY TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_ARBITER"), PHONGO_SERVER_RS_ARBITER TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_OTHER"), PHONGO_SERVER_RS_OTHER TSRMLS_CC);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_GHOST"), PHONGO_SERVER_RS_GHOST TSRMLS_CC);
 
 
 	return SUCCESS;
