@@ -27,7 +27,6 @@
 /* External libs */
 #include <bson.h>
 #include <mongoc.h>
-#include <mongoc-write-concern-private.h>
 
 /* PHP Core stuff */
 #include <php.h>
@@ -47,7 +46,7 @@ PHONGO_API zend_class_entry *php_phongo_writeconcern_ce;
 
 zend_object_handlers php_phongo_handler_writeconcern;
 
-/* {{{ proto MongoDB\Driver\WriteConcern WriteConcern::__construct(integer|string $w[, integer $wtimeout[, boolean $journal]])
+/* {{{ proto void WriteConcern::__construct(integer|string $w[, integer $wtimeout[, boolean $journal]])
    Constructs a new WriteConcern */
 PHP_METHOD(WriteConcern, __construct)
 {
@@ -137,7 +136,7 @@ PHP_METHOD(WriteConcern, getW)
 		PHONGO_RETURN_STRING(PHONGO_WRITE_CONCERN_W_MAJORITY);
 	}
 
-	if (intern->write_concern->w != MONGOC_WRITE_CONCERN_W_DEFAULT) {
+	if (mongoc_write_concern_get_w(intern->write_concern) != MONGOC_WRITE_CONCERN_W_DEFAULT) {
 		RETURN_LONG(mongoc_write_concern_get_w(intern->write_concern));
 	}
 
@@ -175,7 +174,7 @@ PHP_METHOD(WriteConcern, getJournal)
 		return;
 	}
 
-	if (intern->write_concern->journal != MONGOC_WRITE_CONCERN_JOURNAL_DEFAULT) {
+	if (mongoc_write_concern_journal_is_set(intern->write_concern)) {
 		RETURN_BOOL(mongoc_write_concern_get_journal(intern->write_concern));
 	}
 
@@ -183,6 +182,21 @@ PHP_METHOD(WriteConcern, getJournal)
 }
 /* }}} */
 
+/* {{{ proto array WriteConcern::bsonSerialize()
+   */
+PHP_METHOD(WriteConcern, bsonSerialize)
+{
+	const mongoc_write_concern_t *write_concern = phongo_write_concern_from_zval(getThis() TSRMLS_CC);
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	php_phongo_write_concern_to_zval(return_value, write_concern);
+	convert_to_object(return_value);
+}
+/* }}} */
+
 /**
  * Value object for write concern used in issuing write operations.
  */
@@ -192,22 +206,17 @@ ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern___construct, 0, 0, 1)
 	ZEND_ARG_INFO(0, w)
 	ZEND_ARG_INFO(0, wtimeout)
 	ZEND_ARG_INFO(0, journal)
-ZEND_END_ARG_INFO();
+ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_getW, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_getWtimeout, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_getJournal, 0, 0, 0)
-ZEND_END_ARG_INFO();
+ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_void, 0, 0, 0)
+ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_writeconcern_me[] = {
 	PHP_ME(WriteConcern, __construct, ai_WriteConcern___construct, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getW, ai_WriteConcern_getW, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getWtimeout, ai_WriteConcern_getWtimeout, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteConcern, getJournal, ai_WriteConcern_getJournal, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, getW, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, getWtimeout, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, getJournal, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, bsonSerialize, ai_WriteConcern_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_FE_END
 };
 
@@ -280,7 +289,10 @@ PHP_MINIT_FUNCTION(WriteConcern)
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "WriteConcern", php_phongo_writeconcern_me);
 	php_phongo_writeconcern_ce = zend_register_internal_class(&ce TSRMLS_CC);
 	php_phongo_writeconcern_ce->create_object = php_phongo_writeconcern_create_object;
-	PHONGO_CE_INIT(php_phongo_writeconcern_ce);
+	PHONGO_CE_FINAL(php_phongo_writeconcern_ce);
+	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_writeconcern_ce);
+
+	zend_class_implements(php_phongo_writeconcern_ce TSRMLS_CC, 1, php_phongo_serializable_ce);
 
 	memcpy(&php_phongo_handler_writeconcern, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_writeconcern.get_debug_info = php_phongo_writeconcern_get_debug_info;
