@@ -27,7 +27,6 @@
 /* External libs */
 #include <bson.h>
 #include <mongoc.h>
-#include <mongoc-write-concern-private.h>
 
 /* PHP Core stuff */
 #include <php.h>
@@ -51,7 +50,7 @@ PHONGO_API zend_class_entry *php_phongo_writeresult_ce;
 
 zend_object_handlers php_phongo_handler_writeresult;
 
-/* {{{ proto integer WriteResult::getInsertedCount()
+/* {{{ proto integer|null WriteResult::getInsertedCount()
    Returns the number of documents that were inserted */
 PHP_METHOD(WriteResult, getInsertedCount)
 {
@@ -69,7 +68,8 @@ PHP_METHOD(WriteResult, getInsertedCount)
 	RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nInserted");
 }
 /* }}} */
-/* {{{ proto integer WriteResult::getMatchedCount()
+
+/* {{{ proto integer|null WriteResult::getMatchedCount()
    Returns the number of documents that matched the update criteria */
 PHP_METHOD(WriteResult, getMatchedCount)
 {
@@ -87,6 +87,7 @@ PHP_METHOD(WriteResult, getMatchedCount)
 	RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nMatched");
 }
 /* }}} */
+
 /* {{{ proto integer|null WriteResult::getModifiedCount()
    Returns the number of documents that were actually modified by an update */
 PHP_METHOD(WriteResult, getModifiedCount)
@@ -105,7 +106,8 @@ PHP_METHOD(WriteResult, getModifiedCount)
 	RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nModified");
 }
 /* }}} */
-/* {{{ proto integer WriteResult::getDeletedCount()
+
+/* {{{ proto integer|null WriteResult::getDeletedCount()
    Returns the number of documents that were deleted */
 PHP_METHOD(WriteResult, getDeletedCount)
 {
@@ -123,7 +125,8 @@ PHP_METHOD(WriteResult, getDeletedCount)
 	RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nRemoved");
 }
 /* }}} */
-/* {{{ proto integer WriteResult::getUpsertedCount()
+
+/* {{{ proto integer|null WriteResult::getUpsertedCount()
    Returns the number of documents that were upserted */
 PHP_METHOD(WriteResult, getUpsertedCount)
 {
@@ -141,6 +144,7 @@ PHP_METHOD(WriteResult, getUpsertedCount)
 	RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nUpserted");
 }
 /* }}} */
+
 /* {{{ proto MongoDB\Driver\Server WriteResult::getServer()
    Returns the Server from which the result originated */
 PHP_METHOD(WriteResult, getServer)
@@ -162,6 +166,7 @@ PHP_METHOD(WriteResult, getServer)
 #endif
 }
 /* }}} */
+
 /* {{{ proto array WriteResult::getUpsertedIds()
    Returns the identifiers generated by the server for upsert operations. */
 PHP_METHOD(WriteResult, getUpsertedIds)
@@ -224,6 +229,7 @@ PHP_METHOD(WriteResult, getUpsertedIds)
 	}
 }
 /* }}} */
+
 /* {{{ proto WriteConcernError WriteResult::getWriteConcernError()
    Return any write concern error that occurred */
 PHP_METHOD(WriteResult, getWriteConcernError)
@@ -270,6 +276,7 @@ PHP_METHOD(WriteResult, getWriteConcernError)
 	}
 }
 /* }}} */
+
 /* {{{ proto WriteError[] WriteResult::getWriteErrors()
    Returns any write errors that occurred */
 PHP_METHOD(WriteResult, getWriteErrors)
@@ -333,8 +340,10 @@ PHP_METHOD(WriteResult, getWriteErrors)
 	}
 }
 /* }}} */
+
 /* {{{ proto boolean WriteResult::isAcknowledged()
-   Returns the number of documents that were upserted */
+   Returns whether the write operation was acknowledged (based on the write
+   concern). */
 PHP_METHOD(WriteResult, isAcknowledged)
 {
 	php_phongo_writeresult_t *intern;
@@ -348,7 +357,7 @@ PHP_METHOD(WriteResult, isAcknowledged)
 	}
 
 
-	RETURN_BOOL(_mongoc_write_concern_needs_gle(intern->write_concern));
+	RETURN_BOOL(mongoc_write_concern_is_acknowledged(intern->write_concern));
 }
 /* }}} */
 
@@ -360,50 +369,22 @@ PHP_METHOD(WriteResult, isAcknowledged)
  */
 /* {{{ MongoDB\Driver\WriteResult */
 
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getInsertedCount, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getMatchedCount, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getModifiedCount, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getDeletedCount, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getUpsertedCount, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getServer, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getUpsertedIds, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getWriteConcernError, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_getWriteErrors, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
-ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_isAcknowledged, 0, 0, 0)
-ZEND_END_ARG_INFO();
-
+ZEND_BEGIN_ARG_INFO_EX(ai_WriteResult_void, 0, 0, 0)
+ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_writeresult_me[] = {
-	PHP_ME(Server, __construct, NULL, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
-	PHP_ME(WriteResult, getInsertedCount, ai_WriteResult_getInsertedCount, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getMatchedCount, ai_WriteResult_getMatchedCount, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getModifiedCount, ai_WriteResult_getModifiedCount, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getDeletedCount, ai_WriteResult_getDeletedCount, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getUpsertedCount, ai_WriteResult_getUpsertedCount, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getServer, ai_WriteResult_getServer, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getUpsertedIds, ai_WriteResult_getUpsertedIds, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getWriteConcernError, ai_WriteResult_getWriteConcernError, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, getWriteErrors, ai_WriteResult_getWriteErrors, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(WriteResult, isAcknowledged, ai_WriteResult_isAcknowledged, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Manager, __wakeUp, NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(Server, __construct, ai_WriteResult_void, ZEND_ACC_FINAL|ZEND_ACC_PRIVATE)
+	PHP_ME(WriteResult, getInsertedCount, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getMatchedCount, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getModifiedCount, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getDeletedCount, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getUpsertedCount, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getServer, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getUpsertedIds, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getWriteConcernError, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, getWriteErrors, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(WriteResult, isAcknowledged, ai_WriteResult_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Manager, __wakeup, ai_WriteResult_void, ZEND_ACC_PUBLIC)
 	PHP_FE_END
 };
 
@@ -593,7 +574,8 @@ PHP_MINIT_FUNCTION(WriteResult)
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "WriteResult", php_phongo_writeresult_me);
 	php_phongo_writeresult_ce = zend_register_internal_class(&ce TSRMLS_CC);
 	php_phongo_writeresult_ce->create_object = php_phongo_writeresult_create_object;
-	PHONGO_CE_INIT(php_phongo_writeresult_ce);
+	PHONGO_CE_FINAL(php_phongo_writeresult_ce);
+	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_writeresult_ce);
 
 	memcpy(&php_phongo_handler_writeresult, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_writeresult.get_debug_info = php_phongo_writeresult_get_debug_info;
