@@ -115,15 +115,15 @@ bool php_phongo_javascript_has_scope(zval *object TSRMLS_DC)
 
 	intern = Z_JAVASCRIPT_OBJ_P(object);
 
-	return !!intern->document;
+	return !!intern->scope;
 }
-char *php_phongo_javascript_get_javascript(zval *object TSRMLS_DC)
+char *php_phongo_javascript_get_code(zval *object TSRMLS_DC)
 {
 	php_phongo_javascript_t *intern;
 
 	intern = Z_JAVASCRIPT_OBJ_P(object);
 
-	return intern->javascript;
+	return intern->code;
 }
 bson_t *php_phongo_javascript_get_scope(zval *object TSRMLS_DC)
 {
@@ -131,7 +131,7 @@ bson_t *php_phongo_javascript_get_scope(zval *object TSRMLS_DC)
 
 	intern = Z_JAVASCRIPT_OBJ_P(object);
 
-	return intern->document;
+	return intern->scope;
 }
 int php_phongo_binary_get_data(zval *object, char **data TSRMLS_DC)
 {
@@ -150,6 +150,16 @@ uint8_t php_phongo_binary_get_type(zval *object TSRMLS_DC)
 
 	return intern->type;
 }
+
+static bson_decimal128_t *php_phongo_decimal128_get_decimal128(zval *object TSRMLS_DC)
+{
+	php_phongo_decimal128_t *intern;
+
+	intern = Z_DECIMAL128_OBJ_P(object);
+
+	return &intern->decimal;
+}
+
 char *php_phongo_regex_get_pattern(zval *object TSRMLS_DC)
 {
 	php_phongo_regex_t *intern;
@@ -183,7 +193,13 @@ void php_phongo_bson_visit_corrupt(const bson_iter_t *iter ARG_UNUSED, void *dat
 {
 	mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "Corrupt BSON data detected!");
 }
-/* }}} */
+
+void php_phongo_bson_visit_unsupported_type(const bson_iter_t *iter ARG_UNUSED, const char *key, uint32_t v_type_code, void *data ARG_UNUSED) /* {{{ */
+{
+	TSRMLS_FETCH();
+	phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected unknown BSON type 0x%02hhx for fieldname \"%s\". Are you using the latest driver?", v_type_code, key);
+}
+
 bool php_phongo_bson_visit_double(const bson_iter_t *iter ARG_UNUSED, const char *key, double v_double, void *data) /* {{{ */
 {
 #if PHP_VERSION_ID >= 70000
@@ -205,7 +221,7 @@ bool php_phongo_bson_visit_utf8(const bson_iter_t *iter ARG_UNUSED, const char *
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
 
-	ADD_ASSOC_STRING_EX(retval, key, strlen(key), (char *)v_utf8, v_utf8_len);
+	ADD_ASSOC_STRING_EX(retval, key, strlen(key), v_utf8, v_utf8_len);
 
 	return false;
 }
@@ -225,7 +241,7 @@ bool php_phongo_bson_visit_binary(const bson_iter_t *iter ARG_UNUSED, const char
 		zend_class_entry *found_ce = zend_fetch_class(zs_classname, ZEND_FETCH_CLASS_AUTO|ZEND_FETCH_CLASS_SILENT TSRMLS_CC);
 		zend_string_free(zs_classname);
 #else
-		zend_class_entry *found_ce = zend_fetch_class((char *)v_binary, v_binary_len, ZEND_FETCH_CLASS_AUTO|ZEND_FETCH_CLASS_SILENT TSRMLS_CC);
+		zend_class_entry *found_ce = zend_fetch_class((const char *)v_binary, v_binary_len, ZEND_FETCH_CLASS_AUTO|ZEND_FETCH_CLASS_SILENT TSRMLS_CC);
 #endif
 
 		if (found_ce && PHONGO_IS_CLASS_INSTANTIATABLE(found_ce) && instanceof_function(found_ce, php_phongo_persistable_ce TSRMLS_CC)) {
@@ -330,6 +346,31 @@ bool php_phongo_bson_visit_date_time(const bson_iter_t *iter ARG_UNUSED, const c
 	return false;
 }
 /* }}} */
+
+bool php_phongo_bson_visit_decimal128(const bson_iter_t *iter ARG_UNUSED, const char *key, const bson_decimal128_t *decimal, void *data) /* {{{ */
+{
+#if PHP_VERSION_ID >= 70000
+	zval *retval = &((php_phongo_bson_state *)data)->zchild;
+	zval zchild;
+
+	php_phongo_new_decimal128(&zchild, decimal TSRMLS_CC);
+	ADD_ASSOC_ZVAL(retval, key, &zchild);
+#else
+	zval *retval = ((php_phongo_bson_state *)data)->zchild;
+	zval *zchild = NULL;
+	TSRMLS_FETCH();
+
+	MAKE_STD_ZVAL(zchild);
+	php_phongo_new_decimal128(zchild, decimal TSRMLS_CC);
+
+	ADD_ASSOC_ZVAL(retval, key, zchild);
+	Z_SET_REFCOUNT_P(zchild, 1);
+#endif
+
+	return false;
+}
+/* }}} */
+
 bool php_phongo_bson_visit_null(const bson_iter_t *iter ARG_UNUSED, const char *key, void *data) /* {{{ */
 {
 #if PHP_VERSION_ID >= 70000
@@ -489,7 +530,7 @@ bool php_phongo_bson_visit_int64(const bson_iter_t *iter ARG_UNUSED, const char
 #else
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
-
+	TSRMLS_FETCH();
 
 	ADD_ASSOC_INT64(retval, key, v_int64);
 
@@ -574,6 +615,8 @@ static const bson_visitor_t php_bson_visitors = {
    php_phongo_bson_visit_int64,
    php_phongo_bson_visit_maxkey,
    php_phongo_bson_visit_minkey,
+   php_phongo_bson_visit_unsupported_type,
+   php_phongo_bson_visit_decimal128,
    { NULL }
 };
 
@@ -927,6 +970,11 @@ void object_to_bson(zval *object, php_phongo_bson_flags_t flags, const char *key
 			bson_append_binary(bson, key, key_len, php_phongo_binary_get_type(object TSRMLS_CC), data, data_len);
 			return;
 		}
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_decimal128_ce TSRMLS_CC)) {
+			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Decimal128");
+			bson_append_decimal128(bson, key, key_len, php_phongo_decimal128_get_decimal128(object TSRMLS_CC));
+			return;
+		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_regex_ce TSRMLS_CC)) {
 			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Regex");
 			bson_append_regex(bson, key, key_len, php_phongo_regex_get_pattern(object TSRMLS_CC), php_phongo_regex_get_flags(object TSRMLS_CC));
@@ -935,10 +983,10 @@ void object_to_bson(zval *object, php_phongo_bson_flags_t flags, const char *key
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_javascript_ce TSRMLS_CC)) {
 			if (php_phongo_javascript_has_scope(object TSRMLS_CC)) {
 				mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Javascript with scope");
-				bson_append_code(bson, key, key_len, php_phongo_javascript_get_javascript(object TSRMLS_CC));
+				bson_append_code_with_scope(bson, key, key_len, php_phongo_javascript_get_code(object TSRMLS_CC), php_phongo_javascript_get_scope(object TSRMLS_CC));
 			} else {
 				mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Javascript without scope");
-				bson_append_code_with_scope(bson, key, key_len, php_phongo_javascript_get_javascript(object TSRMLS_CC), php_phongo_javascript_get_scope(object TSRMLS_CC));
+				bson_append_code(bson, key, key_len, php_phongo_javascript_get_code(object TSRMLS_CC));
 			}
 			return;
 		}
@@ -964,11 +1012,7 @@ void object_to_bson(zval *object, php_phongo_bson_flags_t flags, const char *key
 			return;
 		}
 
-#if PHP_VERSION_ID >= 70000
 		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Unexpected %s instance: %s", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(object)->name));
-#else
-		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Unexpected %s instance: %s", php_phongo_type_ce->name, Z_OBJCE_P(object)->name);
-#endif
 		return;
 	}
 
@@ -1094,7 +1138,7 @@ static bool is_public_property(zend_class_entry *ce, const char *prop_name, int
 }
 /* }}} */
 
-PHONGO_API void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson, bson_t **bson_out TSRMLS_DC) /* {{{ */
+void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson, bson_t **bson_out TSRMLS_DC) /* {{{ */
 {
 	HashTable   *ht_data = NULL;
 #if PHP_VERSION_ID >= 70000
@@ -1170,11 +1214,7 @@ PHONGO_API void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, b
 			}
 
 			if (instanceof_function(Z_OBJCE_P(data), php_phongo_type_ce TSRMLS_CC)) {
-#if PHP_VERSION_ID >= 70000
 				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s instance %s cannot be serialized as a root element", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(data)->name));
-#else
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s instance %s cannot be serialized as a root element", php_phongo_type_ce->name, Z_OBJCE_P(data)->name);
-#endif
 
 				break;
 			}
@@ -1338,9 +1378,9 @@ PHONGO_API void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, b
 
 /* }}} */
 #if PHP_VERSION_ID >= 70000
-PHONGO_API int phongo_bson_to_zval(const unsigned char *data, int data_len, zval *zv)
+int phongo_bson_to_zval(const unsigned char *data, int data_len, zval *zv)
 #else
-PHONGO_API int phongo_bson_to_zval(const unsigned char *data, int data_len, zval **zv)
+int phongo_bson_to_zval(const unsigned char *data, int data_len, zval **zv)
 #endif
 {
 	int retval = 0;
@@ -1356,7 +1396,7 @@ PHONGO_API int phongo_bson_to_zval(const unsigned char *data, int data_len, zval
 	return retval;
 }
 
-PHONGO_API int phongo_bson_to_zval_ex(const unsigned char *data, int data_len, php_phongo_bson_state *state)
+int phongo_bson_to_zval_ex(const unsigned char *data, int data_len, php_phongo_bson_state *state)
 {
 	      bson_reader_t *reader;
 	      bson_iter_t    iter;
@@ -1399,8 +1439,12 @@ PHONGO_API int phongo_bson_to_zval_ex(const unsigned char *data, int data_len, p
 	if (bson_iter_visit_all(&iter, &php_bson_visitors, state) || iter.err_off) {
 		/* Iteration stopped prematurely due to corruption or a failed visitor.
 		 * While we free the reader, state->zchild should be left as-is, since
-		 * the calling code may want to zval_ptr_dtor() it. */
-		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Could not convert BSON document to a PHP variable");
+		 * the calling code may want to zval_ptr_dtor() it. If an exception has
+		 * been thrown already (due to an unsupported BSON type for example,
+		 * don't overwrite with a generic exception message. */
+		if (!EG(exception)) {
+			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected corrupt BSON data");
+		}
 		bson_reader_destroy(reader);
 		return 0;
 	}
@@ -1500,7 +1544,7 @@ static void apply_classname_to_state(const char *classname, int classname_len, p
 		} else if (!PHONGO_IS_CLASS_INSTANTIATABLE(found_ce)) {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Class %s is not instantiatable", classname);
 		} else if (!instanceof_function(found_ce, php_phongo_unserializable_ce TSRMLS_CC)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Class %s does not implement %s\\Unserializable", classname, BSON_NAMESPACE);
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Class %s does not implement %s", classname, ZSTR_VAL(php_phongo_unserializable_ce->name));
 		} else {
 			*type = PHONGO_TYPEMAP_CLASS;
 			*type_ce = found_ce;
@@ -1508,14 +1552,14 @@ static void apply_classname_to_state(const char *classname, int classname_len, p
 	}
 }
 
-PHONGO_API void phongo_bson_typemap_to_state(zval *typemap, php_phongo_bson_typemap *map TSRMLS_DC)
+void phongo_bson_typemap_to_state(zval *typemap, php_phongo_bson_typemap *map TSRMLS_DC)
 {
 	if (typemap) {
 		char      *classname;
 		int        classname_len;
 		zend_bool  classname_free = 0;
 
-		classname = php_array_fetchl_string(typemap, "array", sizeof("array")-1, &classname_len, &classname_free);
+		classname = php_array_fetchc_string(typemap, "array", &classname_len, &classname_free);
 		if (classname_len) {
 			apply_classname_to_state(classname, classname_len, &map->array_type, &map->array TSRMLS_CC);
 		}
@@ -1523,7 +1567,7 @@ PHONGO_API void phongo_bson_typemap_to_state(zval *typemap, php_phongo_bson_type
 			str_efree(classname);
 		}
 
-		classname = php_array_fetchl_string(typemap, "document", sizeof("document")-1, &classname_len, &classname_free);
+		classname = php_array_fetchc_string(typemap, "document", &classname_len, &classname_free);
 		if (classname_len) {
 			apply_classname_to_state(classname, classname_len, &map->document_type, &map->document TSRMLS_CC);
 		}
@@ -1531,7 +1575,7 @@ PHONGO_API void phongo_bson_typemap_to_state(zval *typemap, php_phongo_bson_type
 			str_efree(classname);
 		}
 
-		classname = php_array_fetchl_string(typemap, "root", sizeof("root")-1, &classname_len, &classname_free);
+		classname = php_array_fetchc_string(typemap, "root", &classname_len, &classname_free);
 		if (classname_len) {
 			apply_classname_to_state(classname, classname_len, &map->root_type, &map->root TSRMLS_CC);
 		}
@@ -1635,7 +1679,7 @@ PHP_FUNCTION(fromJSON)
 		PHONGO_RETVAL_STRINGL((const char *) bson_get_data(&b), b.len);
 		bson_destroy(&b);
 	} else {
-		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s", error.message ? error.message : "Error parsing JSON");
+		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s", error.domain == BSON_ERROR_JSON ? error.message : "Error parsing JSON");
 	}
 }
 /* }}} */
